import { ObservableInput, Subject } from 'rxjs';
import { exhaustMap, switchMap, concatMap, mergeMap, flatMap } from 'rxjs/operators';
interface State<I> {
    data?: I;
    error?: any;
    loading: boolean;
}
export declare type RxAsyncInit<P> = {
    type: 'FETCH_INIT';
    payload?: P;
};
export declare type RxAsyncSuccess<I> = {
    type: 'FETCH_SUCCESS';
    payload: I;
};
export declare type RxAsyncFailure = {
    type: 'FETCH_FAILURE';
    payload?: any;
};
export declare type RxAsyncCancel = {
    type: 'CANCEL';
};
export declare type RxAsyncReset<I> = {
    type: 'RESET';
    payload?: Partial<State<I>>;
};
declare type Actions<I, P> = RxAsyncInit<P> | RxAsyncSuccess<I> | RxAsyncFailure | RxAsyncCancel | RxAsyncReset<I>;
declare type FuncWithParam<P, O> = (params: P) => O;
declare type FuncOptionalParam<P, O> = (() => O) | ((params?: P) => O);
declare type RxAsyncFnWithParam<I, P> = FuncWithParam<P, ObservableInput<I>>;
declare type RxAsyncFnOptionalParam<I, P> = FuncOptionalParam<P, ObservableInput<I>>;
export declare type RxAsyncFn<I, P> = RxAsyncFnWithParam<I, P> | RxAsyncFnOptionalParam<I, P>;
export declare type RxAsyncOptions<I> = {
    defer?: boolean;
    onStart?(): void;
    onSuccess?(value: I): void;
    onFailure?(error: any): void;
    mapOperator?: typeof switchMap | typeof concatMap | typeof exhaustMap | typeof mergeMap | typeof flatMap;
};
export declare type RxAsyncActionsBase<I, P> = {
    subject: Subject<Actions<I, P>>;
    cancel: () => void;
    reset: () => void;
};
export interface RxAsyncActionsWithParam<I, P> extends RxAsyncActionsBase<I, P> {
    fetch: FuncWithParam<P, void>;
}
export interface RxAsyncActionsOptionalParam<I, P> extends RxAsyncActionsBase<I, P> {
    fetch: FuncOptionalParam<P, void>;
}
export declare type RxAsyncState<I> = State<I>;
export declare type RxAsyncActions<I, P> = RxAsyncActionsWithParam<I, P> | RxAsyncActionsOptionalParam<I, P>;
export declare function useRxAsync<I, P>(request: RxAsyncFnWithParam<I, P>, options?: RxAsyncOptions<I> & {
    defer?: boolean;
}): [RxAsyncState<I>, RxAsyncActionsOptionalParam<I, P>];
export declare function useRxAsync<I, P>(request: RxAsyncFnOptionalParam<I, P>, options: RxAsyncOptions<I> & {
    defer: true;
}): [RxAsyncState<I>, RxAsyncActionsWithParam<I, P>];
export {};
