var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { useEffect, useReducer, useRef, useMemo } from 'react';
import { defer as RxDefer, Subject, of } from 'rxjs';
import { switchMap, catchError, takeUntil, filter, tap, map } from 'rxjs/operators';
function initializer(state) {
    return __assign({ loading: false }, state);
}
function reducer(state, action) {
    switch (action.type) {
        case 'FETCH_INIT':
            return __assign(__assign({}, initializer(state)), { loading: true });
        case 'FETCH_SUCCESS':
            return __assign(__assign({}, state), { loading: false, data: action.payload });
        case 'FETCH_FAILURE':
            return __assign(__assign({}, state), { loading: false, error: action.payload });
        case 'CANCEL':
            return __assign(__assign({}, state), { loading: false });
        case 'RESET':
            return initializer(action.payload);
        default:
            throw new Error();
    }
}
var defaultFn = function () { };
export function useRxAsync(request, options) {
    var _a = options || {}, defer = _a.defer, _b = _a.onStart, onStart = _b === void 0 ? defaultFn : _b, _c = _a.onSuccess, onSuccess = _c === void 0 ? defaultFn : _c, _d = _a.onFailure, onFailure = _d === void 0 ? defaultFn : _d, _e = _a.mapOperator, mapOperator = _e === void 0 ? switchMap : _e;
    var _f = useReducer(reducer, { loading: !defer }, initializer), state = _f[0], dispatch = _f[1];
    var actionRef = useRef(new Subject());
    var actions = useMemo(function () {
        return {
            fetch: function (payload) {
                return actionRef.current.next({ type: 'FETCH_INIT', payload: payload });
            },
            reset: function () { return actionRef.current.next({ type: 'RESET' }); },
            cancel: function () { return actionRef.current.next({ type: 'CANCEL' }); },
            subject: actionRef.current
        };
    }, []);
    useEffect(function () {
        var action$ = function () {
            var types = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                types[_i] = arguments[_i];
            }
            return actionRef.current.pipe(filter(function (action) { return types.includes(action.type); }));
        };
        var subscription = action$('FETCH_INIT')
            .pipe(mapOperator(function (action) {
            return RxDefer(function () { return request(action.payload); }).pipe(map(function (payload) { return ({
                type: 'FETCH_SUCCESS',
                payload: payload
            }); }), catchError(function (payload) {
                return of({ type: 'FETCH_FAILURE', payload: payload });
            }), takeUntil(action$('CANCEL', 'RESET')));
        }))
            .subscribe(function (action) {
            actionRef.current.next(action);
        });
        var actionSubscription = actionRef.current
            .pipe(tap(function (action) {
            switch (action.type) {
                case 'FETCH_INIT':
                    return onStart();
                case 'FETCH_SUCCESS':
                    return onSuccess(action.payload);
                case 'FETCH_FAILURE':
                    return onFailure(action.payload);
            }
        }))
            .subscribe(dispatch);
        var fetch = !defer;
        actionRef.current.next({ type: 'RESET', payload: { loading: fetch } });
        if (fetch) {
            actionRef.current.next({ type: 'FETCH_INIT' });
        }
        return function () {
            subscription.unsubscribe();
            actionSubscription.unsubscribe();
        };
    }, [defer, request, actions, mapOperator, onStart, onSuccess, onFailure]);
    return [state, actions];
}
//# sourceMappingURL=useRxAsync.js.map